<?php

namespace Drupal\Tests\sharpspring_webforms\Functional;

use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\webform\Traits\WebformBrowserTestTrait;

/**
 * Tests the sharpspring webforms module.
 *
 * @group sharpspring_webforms
 */
class SharpspringWebformTest extends BrowserTestBase {

  use WebformBrowserTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['sharpspring_webforms'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Strict config checking.
   *
   * @todo disable once
   * https://www.drupal.org/project/sharpspring/issues/3128508 is fixed.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * User with configure webform sharpsrping settings permission.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $adminUser;

  /**
   * A regular user.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $webUser;

  /**
   * Id of the webform.
   *
   * @var int|string|null
   */
  protected $webformId;

  /**
   * The webform storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $webformStorage;

  /**
   * The webform view builder.
   *
   * @var \Drupal\Core\Entity\EntityViewBuilderInterface
   */
  protected $webformViewBuilder;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $this->webUser = $this->createUser();
    $this->adminUser = $this->createUser([
      'administer webform',
      'configure webform sharpspring',
    ]);
    $this->webformId = $this->createWebform()->id();
    $this->webformStorage = $this->container->get('entity_type.manager')->getStorage('webform');
    $this->webformViewBuilder = $this->container->get('entity_type.manager')->getViewBuilder('webform');
  }

  /**
   * Tests the sharpspring webform config form.
   */
  public function testConfigForm() {
    // Check that by default no values are set.
    $webform = $this->webformStorage->loadUnchanged($this->webformId);
    $this->assertEmpty($webform->getThirdPartySettings('sharpspring_webforms'));

    // Unauthorized user should not have access.
    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->assertSession()->statusCodeEquals(403);

    // Login as a regular user.
    $this->drupalLogin($this->webUser);

    // Unauthorized user should not have access.
    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->assertSession()->statusCodeEquals(403);

    // Login as an admin user.
    $this->drupalLogin($this->adminUser);

    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->assertSession()->statusCodeEquals(200);

    // Check if base uri and endpoint are required.
    $this->drupalPostForm(NULL, ['sharpspring_status' => TRUE], 'Save');
    $this->assertSession()->pageTextContains('Base URI field is required.');
    $this->assertSession()->pageTextContains('Endpoint field is required.');
    $this->assertSession()->pageTextNotContains('The configuration options have been saved.');

    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->drupalPostForm(NULL, [
      'sharpspring_status' => TRUE,
      'sharpspring_base_uri' => 'http://random-url.com',
    ], 'Save');
    $this->assertSession()->pageTextNotContains('Base URI field is required.');
    $this->assertSession()->pageTextContains('Endpoint field is required.');
    $this->assertSession()->pageTextNotContains('The configuration options have been saved.');

    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->drupalPostForm(NULL, [
      'sharpspring_status' => TRUE,
      'sharpspring_endpoint' => 'random-endpoint',
    ], 'Save');
    $this->assertSession()->pageTextContains('Base URI field is required.');
    $this->assertSession()->pageTextNotContains('Endpoint field is required.');
    $this->assertSession()->pageTextNotContains('The configuration options have been saved.');

    // Check if saving actually saves the configured values.
    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->drupalPostForm(NULL, [
      'sharpspring_status' => TRUE,
      'sharpspring_base_uri' => 'http://random-url.com',
      'sharpspring_endpoint' => 'random-endpoint',
    ], 'Save');
    $this->assertSession()->pageTextNotContains('Base URI field is required.');
    $this->assertSession()->pageTextNotContains('Endpoint field is required.');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));

    // Check that the values are actually saved.
    $webform = $this->webformStorage->loadUnchanged($this->webformId);
    $this->assertEquals(TRUE, $webform->getThirdPartySetting('sharpspring_webforms', 'status'));
    $this->assertEquals('http://random-url.com', $webform->getThirdPartySetting('sharpspring_webforms', 'base_uri'));
    $this->assertEquals('random-endpoint', $webform->getThirdPartySetting('sharpspring_webforms', 'endpoint'));

    $this->drupalGet(Url::fromRoute('sharpspring_webforms.configuration', ['webform' => $this->webformId]));
    $this->drupalPostForm(NULL, [
      'sharpspring_status' => FALSE,
      'sharpspring_base_uri' => '',
      'sharpspring_endpoint' => '',
    ], 'Save');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Check if removing of the values is working.
    $webform = $this->webformStorage->loadUnchanged($this->webformId);
    $this->assertEmpty($webform->getThirdPartySettings('sharpspring_webforms'));
  }

  /**
   * Tests if the correct drupal settings are set.
   */
  public function testDrupalSettings() {
    // Check that the library and drupalSettings are not present when
    // sharpspring tracking is disabled.
    $webform = $this->webformStorage->loadUnchanged($this->webformId);
    $this->drupalGet($webform->toUrl());
    $drupalSettings = $this->getDrupalSettings();
    $this->assertArrayNotHasKey('sharpspring_webforms', $drupalSettings);
    $this->assertSession()->responseNotContains('sharpspring_webforms/assets/js/sharpspring_webforms.webform.js');

    $webform = $this->webformStorage->loadUnchanged($this->webformId);
    $webform->setThirdPartySetting('sharpspring_webforms', 'status', TRUE);
    $webform->setThirdPartySetting('sharpspring_webforms', 'base_uri', 'http://random-url.com');
    $webform->setThirdPartySetting('sharpspring_webforms', 'endpoint', 'random-endpoint');
    $webform->save();

    // Check that the drupalSettings and library is included when the
    // sharpspring tracking is active.
    $this->drupalGet($webform->toUrl());
    $drupalSettings = $this->getDrupalSettings();
    $this->assertEquals('http://random-url.com', $drupalSettings['sharpspring_webforms']['base_uri']);
    $this->assertEquals('random-endpoint', $drupalSettings['sharpspring_webforms']['endpoint']);
    $this->assertSession()->responseContains('sharpspring_webforms/assets/js/sharpspring_webforms.webform.js');
  }

}
